==> 0.0.2.tar.gz: lib/pages/home.dart <==
import 'dart:async';

import 'package:badges/badges.dart';
import 'package:flutter/material.dart';
import 'package:flutter_typeahead/flutter_typeahead.dart';
import 'package:select_form_field/select_form_field.dart';

import 'package:bizdoc/gen_l10n/app_localizations.dart';
import 'package:bizdoc/core/hub.dart';
import 'package:bizdoc/core/icon.dart';
import 'package:bizdoc/core/extensions.dart';
import 'package:bizdoc/services/accounts.dart';
import 'package:bizdoc/services/base.dart';
import 'package:bizdoc/services/datasource.dart';
import 'package:bizdoc/services/mailbox.dart';
import 'package:bizdoc/services/session.dart';
import 'package:bizdoc/models/models.dart';
import 'package:bizdoc/models/contracts.dart';
import 'package:bizdoc/models/profile.dart';
import 'contacts.dart';
import 'browse.dart';
import 'compose.dart';
import 'conversation.dart';
import 'cube.dart';
import 'dashboard.dart';
import 'drawer.dart';
import 'notifications.dart';

/// Main page.
class HomePage extends StatefulWidget {
  static var routeName = 'home';

  const HomePage({Key? key}) : super(key: key);

  @override
  _HomePageState createState() => _HomePageState();
}

class _HomePageState extends State<HomePage> {
  // badge stream
  final _notificationsController = StreamController<int>.broadcast();
  final _contactsController = StreamController<int>.broadcast();

// menu related vars
  late String? _folderId;
  QueryType? _queryType;
  late CubeInfo? _cube;
  late CubeView? _cubeView;
  final Map<String, dynamic> _filters = {};

  // creatable forms iterable
  late Iterable<FormInfo> _forms;

  ContactInfo? _chatting;

  late Iterable<ActionInfo> _actions = [];

  @override
  void initState() {
    _forms = session.profile.forms.where((element) => element.creatable);
    _listen();
    _chatting = null;
    _folderId = FolderInfo.inbox;
    _cube = null;
    _cubeView = null;
    super.initState();
  }

  /// listen to hub events and update notifications and conversations counter badge
  Future<void> _listen() async {
    _notificationsController.add(session.profile.notificationsCount);
    _contactsController.add(session.profile.conversationsCount);
    await for (var e in hub.events) {
      if (e is NewChat) {
        var contact = session.profile.contacts
            .firstWhereOrNull((element) => element.id == e.contactId);
        if (contact == null) {
          final info = await accounts.userInfo(e.contactId);
          contact = ContactInfo.from(info);
          session.profile.contacts.insert(0, contact);
          contact.count = 1;
          _contactsController.add(session.profile.conversationsCount++);
        } else {
          contact.count++;
          if (contact.count > 1) {
            _contactsController.add(session.profile.conversationsCount++);
          }
        }
      } else if (e is NewMail) {
        try {
          ScaffoldMessenger.of(context).showSnackBar(SnackBar(
            content: Text('#${e.model.number}'),
            action: SnackBarAction(
              label: "Open",
              onPressed: () {},
            ),
          ));
        } catch (e) {
          print(e);
        }
      } else if (e is NewNotify) {
        session.profile.notifications.insert(0, e.model);
        _notificationsController.add(session.profile.notificationsCount++);
      }
    }
  }

  @override
  Widget build(BuildContext context) {
    Widget body;
    if (_folderId != null || _queryType != null) {
      body = BrowsePage(
        folderId: _folderId,
        queryType: _queryType,
        onSelected: (actions) {
          setState(() {
            _actions = actions;
          });
        },
      );
    } else if (_cube != null) {
      body = CubeViewPage(cube: _cube!, view: _cubeView!, filters: _filters);
    } else {
      body = const DashboardPage();
    }

    return Scaffold(
      appBar: AppBar(
        title: Text(AppLocalizations.of(context)!.appTitle),
        actions: [
          if (_actions.isNotEmpty)
            IconButton(onPressed: _sendAll, icon: const Icon(Icons.send)),
          StreamBuilder<int?>(
              stream: _notificationsController.stream,
              builder: (context, snapshot) => Badge(
                    child: IconButton(
                      icon: const Icon(Icons.notification_add),
                      onPressed: () {
                        Navigator.pushNamed(
                            context, NotificationPage.routeName);
                        session.seenNotifications();
                      },
                    ),
                    badgeColor: Theme.of(context).colorScheme.secondary,
                    badgeContent: Text(snapshot.data.toString()),
                    showBadge: snapshot.hasData && snapshot.data! > 0,
                    position: const BadgePosition(top: 0),
                  )),
          StreamBuilder<int?>(
              stream: _contactsController.stream,
              builder: (context, snapshot) => Badge(
                    child: IconButton(
                      icon: const Icon(Icons.chat),
                      onPressed: () => Scaffold.of(context).openEndDrawer(),
                    ),
                    badgeColor: Theme.of(context).colorScheme.secondary,
                    badgeContent: Text(snapshot.data.toString()),
                    showBadge: snapshot.hasData && snapshot.data! > 0,
                    position: const BadgePosition(top: 0),
                  )),
        ],
      ),
      body: body,
      // return Navigator(pages: [
      //   MaterialPage(key: ValueKey(''), child:
      // ())
      // ],);
      drawer: Drawer(
          child: MainDrawer(
        onTap: (item) => _nav(item),
      )),
      floatingActionButton: FloatingActionButton(
        onPressed: () {
          if (_cube != null) {
            showFilter();
          } else {
            _compose();
          }
        },
        child: _cubeView != null
            ? const Icon(Icons.filter_list)
            : const Icon(Icons.add),
      ),
      endDrawer: Drawer(
          child: (_chatting == null)
              ? ContactsWidget(
                  onTap: (e) {
                    setState(() {
                      _chatting = e;
                    });
                  },
                )
              : ConversationWidget(chatWith: _chatting!)),
    );
  }

  void _nav(dynamic item) {
    setState(() {
      if (item is FolderEvent) {
        _folderId = item.folderId;
        _queryType = item.queryType;
        _cube = null;
        _cubeView = null;
      } else if (item is CubeEvent) {
        _folderId = null;
        _queryType = null;
        _cube = item.cube;
        _cubeView = item.view;
      } else {
        _folderId = null;
        _queryType = null;
        _cube = null;
        _cubeView = null;
      }
    });
  }

  void _compose() async {
    if (_forms.length == 1) {
      await _create(_forms.first);
    } else {
      showModalBottomSheet(
        context: context,
        builder: (context) {
          return ListView.builder(
            itemCount: _forms.length,
            itemBuilder: (context, index) {
              final form = _forms.elementAt(index);
              return ListTile(
                title: Text(form.title),
                leading: Icon(iconData(form.icon)),
                onTap: () {
                  Navigator.pop(context);
                  _create(form);
                },
              );
            },
          );
        },
      );
    }
  }

  Future<void> _create(FormInfo form) async {
    double? lat, lng;
    switch (form.position) {
      case PositionPolicy.require:
      case PositionPolicy.ask:
        final location = await session.geoLocation();
        if (location != null) {
          lat = location.latitude;
          lng = location.latitude;
        } else if (form.position == PositionPolicy.require) {
          ScaffoldMessenger.of(context).showSnackBar(
              SnackBar(content: Text(AppLocalizations.of(context)!.err)));
          return;
        }
        break;
      default:
    }
    try {
      final model =
          await mailbox.create(form.name, latitude: lat, longitude: lng);

      Navigator.pushNamed(context, ComposePage.routeName,
          arguments: ComposeModel(model));
    } catch (e) {
      if (e is UnauthorizedError) {
      } else {
        ScaffoldMessenger.of(context).showSnackBar(SnackBar(
          content: Text(AppLocalizations.of(context)!.err),
        ));
      }
    }
  }

  _sendAll() {
    if (_actions.length == 1) {
      _action(_actions.elementAt(0));
    } else {
      showBottomSheet(
        context: context,
        builder: (context) {
          return ListView(
            children: [
              for (var action in _actions)
                ListTile(
                  title: Text(action.name),
                  onTap: () {
                    Navigator.pop(context);
                    _action(action);
                  },
                )
            ],
          );
        },
      );
    }
  }

  void showFilter() {
    void setFilter(String name, String? value) {
      _filters[name] = value;
      setState(() {});
    }

    showModalBottomSheet(
      context: context,
      builder: (context) {
        final children = <Widget>[];
        final axes = _cube!.axes.where((element) => element.combination);
        for (var item in axes) {
          switch (item.selectionMode) {
            case SelectionMode.pattern:
              children.add(TextField(
                decoration: InputDecoration(labelText: item.title),
                onChanged: (value) {
                  setFilter(item.name, value);
                },
              ));
              break;
            case SelectionMode.single:
            case SelectionMode.multiple:
              children.add(FutureBuilder<Map<String, dynamic>>(
                  future: dataSource.getMap(item.dataType),
                  builder: (context, snapshot) {
                    if (snapshot.hasData) {
                      return snapshot.data!.length > 10
                          ? TypeAheadFormField<MapEntry>(
                              onSuggestionSelected: (suggestion) {
                                setFilter(item.name, suggestion.key);
                              },
                              itemBuilder: (context, itemData) =>
                                  ListTile(title: Text(itemData.value)),
                              suggestionsCallback: (pattern) {
                                final value = pattern.toLowerCase();
                                return snapshot.data!.entries.where((element) =>
                                    element.key
                                        .toLowerCase()
                                        .startsWith(value) ||
                                    element.value
                                        .toString()
                                        .toLowerCase()
                                        .contains(value));
                              },
                            )
                          : SelectFormField(
                              decoration: InputDecoration(
                                  labelText: item.title ?? item.name),
                              type: SelectFormFieldType.dropdown,
                              initialValue: null,
                              items: snapshot.data!.entries
                                  .map(
                                      (e) => {'value': e.key, 'label': e.value})
                                  .toList(),
                              onChanged: (value) => setFilter(item.name, value),
                            );
                    }
                    return Container();
                  }));
              break;
            default:
          }
        }
        return SingleChildScrollView(
          child: Column(children: children),
        );
      },
    );
  }

  void _action(ActionInfo elementAt) {}
}

==> 0.0.3.tar.gz: lib/pages/home.dart <==
import 'dart:async';

import 'package:badges/badges.dart';
import 'package:flutter/material.dart';
import 'package:flutter_typeahead/flutter_typeahead.dart';
import 'package:select_form_field/select_form_field.dart';

import 'package:bizdoc/gen_l10n/app_localizations.dart';
import 'package:bizdoc/core/hub.dart';
import 'package:bizdoc/core/icon.dart';
import 'package:bizdoc/core/extensions.dart';
import 'package:bizdoc/services/accounts.dart';
import 'package:bizdoc/services/base.dart';
import 'package:bizdoc/services/datasource.dart';
import 'package:bizdoc/services/mailbox.dart';
import 'package:bizdoc/services/session.dart';
import 'package:bizdoc/models/data.dart';
import 'package:bizdoc/models/contracts.dart';
import 'package:bizdoc/models/profile.dart';
import 'contacts.dart';
import 'browse.dart';
import 'compose.dart';
import 'conversation.dart';
import 'cube.dart';
import 'dashboard.dart';
import 'drawer.dart';
import 'notifications.dart';

/// Main page.
class HomePage extends StatefulWidget {
  static var routeName = 'home';

  const HomePage({Key? key}) : super(key: key);

  @override
  _HomePageState createState() => _HomePageState();
}

class _HomePageState extends State<HomePage> {
  // badge stream
  final _notificationsController = StreamController<int>.broadcast();
  final _contactsController = StreamController<int>.broadcast();

// menu related vars
  late String? _folderId;
  QueryType? _queryType;
  late CubeInfo? _cube;
  late CubeView? _cubeView;
  final Map<String, dynamic> _filters = {};

  // creatable forms iterable
  late Iterable<FormInfo> _forms;

  ContactInfo? _chatting;

  late Iterable<ActionInfo> _actions = [];

  @override
  void initState() {
    _forms = session.profile.forms.where((element) => element.creatable);
    _listen();
    _chatting = null;
    _folderId = FolderInfo.inbox;
    _cube = null;
    _cubeView = null;
    super.initState();
  }

  /// listen to hub events and update notifications and conversations counter badge
  Future<void> _listen() async {
    _notificationsController.add(session.profile.notificationsCount);
    _contactsController.add(session.profile.conversationsCount);
    await for (var e in hub.events) {
      if (e is NewChat) {
        var contact = session.profile.contacts
            .firstWhereOrNull((element) => element.id == e.contactId);
        if (contact == null) {
          final info = await accounts.find(e.contactId);
          contact = ContactInfo.from(info);
          session.profile.contacts.insert(0, contact);
          contact.count = 1;
          _contactsController.add(session.profile.conversationsCount++);
        } else {
          contact.count++;
          if (contact.count > 1) {
            _contactsController.add(session.profile.conversationsCount++);
          }
        }
      } else if (e is NewMail) {
        try {
          ScaffoldMessenger.of(context).showSnackBar(SnackBar(
            content: Text('#${e.model.number}'),
            action: SnackBarAction(
              label: "Open",
              onPressed: () {},
            ),
          ));
        } catch (e) {
          print(e);
        }
      } else if (e is NewNotify) {
        session.profile.notifications.insert(0, e.model);
        _notificationsController.add(session.profile.notificationsCount++);
      }
    }
  }

  @override
  Widget build(BuildContext context) {
    Widget body;
    if (_folderId != null || _queryType != null) {
      body = BrowsePage(
        folderId: _folderId,
        queryType: _queryType,
        onSelected: (actions) {
          setState(() {
            _actions = actions;
          });
        },
      );
    } else if (_cube != null) {
      body = CubeViewPage(cube: _cube!, view: _cubeView!, filters: _filters);
    } else {
      body = const DashboardPage();
    }

    return Scaffold(
      appBar: AppBar(
        title: Text(AppLocalizations.of(context)!.appTitle),
        actions: [
          if (_actions.isNotEmpty)
            IconButton(onPressed: _sendAll, icon: const Icon(Icons.send)),
          StreamBuilder<int?>(
              stream: _notificationsController.stream,
              builder: (context, snapshot) => Badge(
                    child: IconButton(
                      icon: const Icon(Icons.notification_add),
                      onPressed: () {
                        Navigator.pushNamed(
                            context, NotificationPage.routeName);
                        session.seenNotifications();
                      },
                    ),
                    badgeColor: Theme.of(context).colorScheme.secondary,
                    badgeContent: Text(snapshot.data.toString()),
                    showBadge: snapshot.hasData && snapshot.data! > 0,
                    position: const BadgePosition(top: 0),
                  )),
          StreamBuilder<int?>(
              stream: _contactsController.stream,
              builder: (context, snapshot) => Badge(
                    child: IconButton(
                      icon: const Icon(Icons.chat),
                      onPressed: () => Scaffold.of(context).openEndDrawer(),
                    ),
                    badgeColor: Theme.of(context).colorScheme.secondary,
                    badgeContent: Text(snapshot.data.toString()),
                    showBadge: snapshot.hasData && snapshot.data! > 0,
                    position: const BadgePosition(top: 0),
                  )),
        ],
      ),
      body: body,
      // return Navigator(pages: [
      //   MaterialPage(key: ValueKey(''), child:
      // ())
      // ],);
      drawer: Drawer(
          child: MainDrawer(
        onTap: (item) => _nav(item),
      )),
      floatingActionButton: FloatingActionButton(
        onPressed: () {
          if (_cube != null) {
            showFilter();
          } else {
            _compose();
          }
        },
        child: _cubeView != null
            ? const Icon(Icons.filter_list)
            : const Icon(Icons.add),
      ),
      endDrawer: Drawer(
          child: (_chatting == null)
              ? ContactsWidget(
                  onTap: (e) {
                    setState(() {
                      _chatting = e;
                    });
                  },
                )
              : ConversationWidget(chatWith: _chatting!)),
    );
  }

  void _nav(dynamic item) {
    setState(() {
      if (item is FolderEvent) {
        _folderId = item.folderId;
        _queryType = item.queryType;
        _cube = null;
        _cubeView = null;
      } else if (item is CubeEvent) {
        _folderId = null;
        _queryType = null;
        _cube = item.cube;
        _cubeView = item.view;
      } else {
        _folderId = null;
        _queryType = null;
        _cube = null;
        _cubeView = null;
      }
    });
  }

  void _compose() async {
    if (_forms.length == 1) {
      await _create(_forms.first);
    } else {
      showModalBottomSheet(
        context: context,
        builder: (context) {
          return ListView.builder(
            itemCount: _forms.length,
            itemBuilder: (context, index) {
              final form = _forms.elementAt(index);
              return ListTile(
                title: Text(form.title),
                leading: Icon(iconData(form.icon)),
                onTap: () {
                  Navigator.pop(context);
                  _create(form);
                },
              );
            },
          );
        },
      );
    }
  }

  Future<void> _create(FormInfo form) async {
    double? lat, lng;
    switch (form.position) {
      case PositionPolicy.require:
      case PositionPolicy.ask:
        final location = await session.geoLocation();
        if (location != null) {
          lat = location.latitude;
          lng = location.latitude;
        } else if (form.position == PositionPolicy.require) {
          ScaffoldMessenger.of(context).showSnackBar(
              SnackBar(content: Text(AppLocalizations.of(context)!.err)));
          return;
        }
        break;
      default:
    }
    try {
      final model =
          await mailbox.create(form.name, latitude: lat, longitude: lng);

      Navigator.pushNamed(context, ComposePage.routeName,
          arguments: ComposeModel(model));
    } catch (e) {
      if (e is UnauthorizedError) {
      } else {
        ScaffoldMessenger.of(context).showSnackBar(SnackBar(
          content: Text(AppLocalizations.of(context)!.err),
        ));
      }
    }
  }

  _sendAll() {
    if (_actions.length == 1) {
      _action(_actions.elementAt(0));
    } else {
      showBottomSheet(
        context: context,
        builder: (context) {
          return ListView(
            children: [
              for (var action in _actions)
                ListTile(
                  title: Text(action.name),
                  onTap: () {
                    Navigator.pop(context);
                    _action(action);
                  },
                )
            ],
          );
        },
      );
    }
  }

  void showFilter() {
    void setFilter(String name, String? value) {
      _filters[name] = value;
      setState(() {});
    }

    showModalBottomSheet(
      context: context,
      builder: (context) {
        final children = <Widget>[];
        final axes = _cube!.axes.where((element) => element.combination);
        for (var item in axes) {
          switch (item.selectionMode) {
            case SelectionMode.pattern:
              children.add(TextField(
                decoration: InputDecoration(labelText: item.title),
                onChanged: (value) {
                  setFilter(item.name, value);
                },
              ));
              break;
            case SelectionMode.single:
            case SelectionMode.multiple:
              children.add(FutureBuilder<Map<String, dynamic>>(
                  future: dataSource.map(item.dataType),
                  builder: (context, snapshot) {
                    if (snapshot.hasData) {
                      return snapshot.data!.length > 10
                          ? TypeAheadFormField<MapEntry>(
                              onSuggestionSelected: (suggestion) {
                                setFilter(item.name, suggestion.key);
                              },
                              itemBuilder: (context, itemData) =>
                                  ListTile(title: Text(itemData.value)),
                              suggestionsCallback: (pattern) {
                                final value = pattern.toLowerCase();
                                return snapshot.data!.entries.where((element) =>
                                    element.key
                                        .toLowerCase()
                                        .startsWith(value) ||
                                    element.value
                                        .toString()
                                        .toLowerCase()
                                        .contains(value));
                              },
                            )
                          : SelectFormField(
                              decoration: InputDecoration(
                                  labelText: item.title ?? item.name),
                              type: SelectFormFieldType.dropdown,
                              initialValue: null,
                              items: snapshot.data!.entries
                                  .map(
                                      (e) => {'value': e.key, 'label': e.value})
                                  .toList(),
                              onChanged: (value) => setFilter(item.name, value),
                            );
                    }
                    return Container();
                  }));
              break;
            default:
          }
        }
        return SingleChildScrollView(
          child: Column(children: children),
        );
      },
    );
  }

  void _action(ActionInfo elementAt) {}
}
